@@ -410,7 +410,7 @@ check_arrays <- function(list1, y) {
 
 # Check arguments from the BH_SBVAR function
 #' @keywords internal
-arguments_check <- function(y, nlags, pA, pdetA, pH, pP, pP_sig, pR_sig, kappa1, itr, burn, thin, acc_irf, h1_irf, ci, cri, rA, rB) {
+arguments_check <- function(y, nlags, pA, pdetA, pH, pP, pP_sig, pR_sig, kappa1, itr, burn, thin, acc_irf, h1_irf, ci, cri, rA, rB, rD) {
   test <- check_integers(list1 = list(nlags = nlags, itr = itr, burn = burn, thin = thin, h1_irf = h1_irf))
   if (test != "pass") {
     return(test)
@@ -485,7 +485,10 @@ arguments_check <- function(y, nlags, pA, pdetA, pH, pP, pP_sig, pR_sig, kappa1,
   }
   if ((!isTRUE(rB)) & (!isFALSE(rB))) {
     return("rB: Should be TRUE or FALSE.")
-  }  
+  }
+  if ((!isTRUE(rD)) & (!isFALSE(rD))) {
+    return("rD: Should be TRUE or FALSE.")
+  }
   return("pass")
 }
 
@@ -548,16 +551,17 @@ acf_plot <- function(data1, prior_name, i, j) {
 #' @param cri credibility intervals for the estimates to be returned (default = 0.95). A value of 0.95 will return 95\% credibility intervals. A value of 0.90 will return 90\% credibility intervals.
 #' @param rA Boolean indicating whether raw results for \emph{A} should be returned (default = FALSE).
 #' @param rB Boolean indicating whether raw results for \emph{B} should be returned (default = FALSE).
+#' @param rD Boolean indicating whether raw results for \emph{D} should be returned (default = FALSE).
 #' @details Estimates the parameters of a Structural Bayesian Vector Autoregression model with the method developed in Baumeister and Hamilton (2015/2017/2018). The function returns a list containing the results.
 #' @return A list containing the following:
 #' @return accept_rate: Acceptance rate of the algorithm.
 #' @return y and x: Matrices containing the endogenous variables and their lags.
 #' @return pA, pdetA, pH, pP, pP_sig, pR, pR_sig, tau1, and kappa1: Matrices and arrays containing prior information.
 #' @return A_start: Matrix containing estimates of the parameters in \emph{A} from the optimization routine.
-#' @return A, detA, H, B, and Phi: Arrays containing estimates of the model parameters. The first, second, and third slices of the third dimension are lower, median, and upper bounds of the estimates.
-#' @return HD and IRF: Arrays containing historical decompositions and impulse response functions. The first, second, and third slices of the third dimension are lower, median, and upper bounds of the estimates.
+#' @return A, detA, H, B, Phi, and D: Arrays containing estimates of the model parameters. The first, second, and third slices of the third dimension are lower, median, and upper bounds of the estimates.
+#' @return HD, IRF, FEVD: Arrays containing historical decompositions, impulse response functions, and variance decompositions. The first, second, and third slices of the third dimension are lower, median, and upper bounds of the estimates.
 #' @return A_den, detA_den, and H_den: Lists containing the horizontal and vertical axis coordinates of posterior densities of \emph{A}, \emph{det(A)}, and \emph{H}.
-#' @return A_chain and B_chain: Arrays containing the raw results for \emph{A} and \emph{B}.
+#' @return A_chain, B_chain, D_chain: Arrays containing the raw results for \emph{A}, \emph{B}, \emph{D}.
 #' @return Line and ACF plots of the estimates for \emph{A}, \emph{det(A)}, and \emph{H}.
 #' @references Baumeister, C., & Hamilton, J.D. (2015). Sign restrictions, structural vector autoregressions, and useful prior information. \emph{Econometrica}, 83(5), 1963-1999.
 #' @references Baumeister, C., & Hamilton, J.D. (2017). Structural interpretation of vector autoregressions with incomplete identification: Revisiting the role of oil supply and demand shocks (No. w24167). National Bureau of Economic Research.
@@ -661,7 +665,7 @@ acf_plot <- function(data1, prior_name, i, j) {
 #'            pR_sig = pR_sig, kappa1 = kappa1, itr = itr, burn = burn,
 #'            thin = thin, acc_irf = acc_irf,
 #'            h1_irf = h1_irf, cri = cri)
-BH_SBVAR <- function(y, nlags, pA, pdetA = NULL, pH = NULL, pP = NULL, pP_sig = NULL, pR_sig = NULL, kappa1 = NULL, itr = 5000, burn = 0, thin = 1, acc_irf = TRUE, h1_irf = 12, ci = NULL, cri = 0.95, rA = FALSE, rB = FALSE) {
+BH_SBVAR <- function(y, nlags, pA, pdetA = NULL, pH = NULL, pP = NULL, pP_sig = NULL, pR_sig = NULL, kappa1 = NULL, itr = 5000, burn = 0, thin = 1, acc_irf = TRUE, h1_irf = 12, ci = NULL, cri = 0.95, rA = FALSE, rB = FALSE, rD = FALSE) {
   
   #construct objects from NULL inputs
   if (is.null(pdetA)) {
@@ -682,7 +686,7 @@ BH_SBVAR <- function(y, nlags, pA, pdetA = NULL, pH = NULL, pP = NULL, pP_sig =
   }
   
   #check BH_SBVAR function arguments
-  test <- arguments_check(y = y, nlags = nlags, pA = pA, pdetA = pdetA, pH = pH, pP = pP, pP_sig = pP_sig, pR_sig = pR_sig, kappa1 = kappa1, itr = itr, burn = burn, thin = thin, acc_irf = acc_irf, h1_irf = h1_irf, ci = ci, cri = cri, rA = rA, rB = rB)
+  test <- arguments_check(y = y, nlags = nlags, pA = pA, pdetA = pdetA, pH = pH, pP = pP, pP_sig = pP_sig, pR_sig = pR_sig, kappa1 = kappa1, itr = itr, burn = burn, thin = thin, acc_irf = acc_irf, h1_irf = h1_irf, ci = ci, cri = cri, rA = rA, rB = rB, rD = rD)
   if (test != "pass") {
     stop(test)
   }
@@ -775,7 +779,7 @@ BH_SBVAR <- function(y, nlags, pA, pdetA = NULL, pH = NULL, pP = NULL, pP_sig =
   scale1 <- PH * scale_ar
   
   #Metropolis-Hastings Algorithm
-  results <- MAIN(y1 = y1, x1 = x1, omega = omega, somega = somega, nlags = nlags, pA = pA, pdetA = pdetA, pH = pH, pP = pP, pP_sig = pP_sig, pR_sig = pR_sig, kappa1 = kappa1, A_start = A_start, itr = itr, burn = burn, thin = thin, scale1 = scale1, h1_irf = h1_irf, acc_irf = acc_irf, ci = ci, varnames = varnames, line_plot = line_plot, acf_plot = acf_plot, rA = rA, rB = rB)
+  results <- MAIN(y1 = y1, x1 = x1, omega = omega, somega = somega, nlags = nlags, pA = pA, pdetA = pdetA, pH = pH, pP = pP, pP_sig = pP_sig, pR_sig = pR_sig, kappa1 = kappa1, A_start = A_start, itr = itr, burn = burn, thin = thin, scale1 = scale1, h1_irf = h1_irf, acc_irf = acc_irf, ci = ci, varnames = varnames, line_plot = line_plot, acf_plot = acf_plot, rA = rA, rB = rB, rD = rD)
   
   dimnames(results$y) <- dimnames(y1)
   dimnames(results$x) <- dimnames(x1)
@@ -801,9 +805,11 @@ BH_SBVAR <- function(y, nlags, pA, pdetA = NULL, pH = NULL, pP = NULL, pP_sig =
   
   dimnames(results$B) <- list(colnames(x1), paste0(colnames(y1), "_Eq"), paste0(c(((1 - ci) * 100), 50, (ci * 100)),"%"))
   dimnames(results$Phi) <- list(colnames(x1), paste0(colnames(y1), "_Eq"), paste0(c(((1 - ci) * 100), 50, (ci * 100)),"%"))
+  dimnames(results$D) <- list(varnames, paste0(varnames, "_Eq"), paste0(c(((1 - ci) * 100), 50, (ci * 100)),"%"))
   
   dimnames(results$HD) <- list(NULL, paste0("Res_", varnames, paste0("_Shk_", varnames[c(sort(x = rep(x = c(1:ncol(y1)), times = ncol(y1))))])), paste0(c(((1 - ci) * 100), 50, (ci * 100)),"%"))
   dimnames(results$IRF) <- list(NULL, paste0("Res_", varnames, paste0("_Shk_", varnames[c(sort(x = rep(x = c(1:ncol(y1)), times = ncol(y1))))])), paste0(c(((1 - ci) * 100), 50, (ci * 100)),"%"))
+  dimnames(results$FEVD) <- list(NULL, paste0("Res_", varnames, paste0("_Shk_", varnames[c(sort(x = rep(x = c(1:ncol(y1)), times = ncol(y1))))])), paste0(c(((1 - ci) * 100), 50, (ci * 100)),"%"))
   
   dimnames(results$A_den$hori) <- list(varnames, paste0(varnames, "_Eq"), NULL)
   dimnames(results$A_den$vert) <- list(varnames, paste0(varnames, "_Eq"), NULL)
@@ -818,6 +824,9 @@ BH_SBVAR <- function(y, nlags, pA, pdetA = NULL, pH = NULL, pP = NULL, pP_sig =
   if (rB) {
     dimnames(results$B_chain) <- list(colnames(x1), paste0(varnames, "_Eq"), NULL)
   }
+  if (rD) {
+    dimnames(results$D_chain) <- list(varnames, paste0(varnames, "_Eq"), NULL)
+  }
   
   return(results)
 }
@@ -837,6 +846,9 @@ check_results <- function(results, xlab, ylab) {
   if ((is.null(results$IRF)) || (!is.array(results$IRF)) || (any(!is.finite(results$IRF))) || (dim(results$IRF)[1] < 4) || (dim(results$IRF)[3] < 3) || (dim(results$IRF)[2] != ((dim(results$A)[2])^2))) {
     return(paste("results: IRF from BH_SBVAR() is not present", sep = ""))
   }
+  if ((is.null(results$FEVD)) || (!is.array(results$FEVD)) || (any(!is.finite(results$FEVD))) || (dim(results$FEVD)[1] < 4) || (dim(results$FEVD)[3] < 3) || (dim(results$FEVD)[2] != ((dim(results$A)[2])^2))) {
+    return(paste("results: FEVD from BH_SBVAR() is not present", sep = ""))
+  }
   if ((is.null(results$HD)) || (!is.array(results$HD)) || (dim(results$HD)[2] < 4) || (dim(results$HD)[3] < 3) || (dim(results$HD)[2] != ((dim(results$A)[2])^2))) {
     return(paste("results: HD from BH_SBVAR() is not present", sep = ""))
   }
@@ -1012,6 +1024,191 @@ IRF_Plots <- function(results, varnames, shocknames = NULL, xlab = NULL, ylab =
   return(irf_results)
 }
 
+#' Plot Forecast Error Variance Decompositions
+#' 
+#' Plot Forecast Error Variance Decompositions
+#' @author Paul Richardson
+#' @export
+#' @name FEVD_Plots
+#' @param results List containing the results from running BH_SBVAR().
+#' @param varnames Character vector containing the names of the endogenous variables.
+#' @param shocknames Character vector containing the names of the shocks.
+#' @param xlab Character label for the horizontal axis of impulse response plots (default = NULL). Default produces plots without a label for the horizontal axis.
+#' @param ylab Character label for the vertical axis of impulse response plots (default = NULL). Default produces plots without a label for the vertical axis.
+#' @param rel Boolean indicating whether to display forecast error variance explained by the shock as a percent of total forecast error variance (default = TRUE).
+#' @details Plots impulse responses and returns a list containing the actual processed data used to create the plots.
+#' @return A list containing impulse responses.
+#' @examples
+#' # Import data
+#' library(BHSBVAR)
+#' set.seed(123)
+#' data(USLMData)
+#' y0 <- matrix(data = c(USLMData$Wage, USLMData$Employment), ncol = 2)
+#' y <- y0 - (matrix(data = 1, nrow = nrow(y0), ncol = ncol(y0)) %*% 
+#'              diag(x = colMeans(x = y0, na.rm = FALSE, dims = 1)))
+#' colnames(y) <- c("Wage", "Employment")
+#' 
+#' # Set function arguments
+#' nlags <- 8
+#' itr <- 5000
+#' burn <- 0
+#' thin <- 1
+#' acc_irf <- TRUE
+#' h1_irf <- 20
+#' cri <- 0.95
+#' 
+#' # Priors for A
+#' pA <- array(data = NA, dim = c(2, 2, 8))
+#' pA[, , 1] <- c(0, NA, 0, NA)
+#' pA[, , 2] <- c(1, NA, -1, NA)
+#' pA[, , 3] <- c(0.6, 1, -0.6, 1)
+#' pA[, , 4] <- c(0.6, NA, 0.6, NA)
+#' pA[, , 5] <- c(3, NA, 3, NA)
+#' pA[, , 6] <- c(NA, NA, NA, NA)
+#' pA[, , 7] <- c(NA, NA, 1, NA)
+#' pA[, , 8] <- c(2, NA, 2, NA)
+#' 
+#' # Position priors for Phi
+#' pP <- matrix(data = 0, nrow = ((nlags * ncol(pA)) + 1), ncol = ncol(pA))
+#' pP[1:nrow(pA), 1:ncol(pA)] <-
+#'   diag(x = 1, nrow = nrow(pA), ncol = ncol(pA))
+#' 
+#' # Confidence in the priors for Phi
+#' x1 <- 
+#'   matrix(data = NA, nrow = (nrow(y) - nlags), 
+#'          ncol = (ncol(y) * nlags))
+#' for (k in 1:nlags) {
+#'   x1[, (ncol(y) * (k - 1) + 1):(ncol(y) * k)] <-
+#'     y[(nlags - k + 1):(nrow(y) - k),]
+#' }
+#' x1 <- cbind(x1, 1)
+#' colnames(x1) <- 
+#'   c(paste(rep(colnames(y), nlags),
+#'           "_L",
+#'           sort(rep(seq(from = 1, to = nlags, by = 1), times = ncol(y)),
+#'                decreasing = FALSE),
+#'           sep = ""),
+#'     "cons")
+#' y1 <- y[(nlags + 1):nrow(y),]
+#' ee <- matrix(data = NA, nrow = nrow(y1), ncol = ncol(y1))
+#' for (i in 1:ncol(y1)) {
+#'   xx <- cbind(x1[, seq(from = i, to = (ncol(x1) - 1), by = ncol(y1))], 1)
+#'   yy <- matrix(data = y1[, i], ncol = 1)
+#'   phi <- solve(t(xx) %*% xx, t(xx) %*% yy)
+#'   ee[, i] <- yy - (xx %*% phi)
+#' }
+#' somega <- (t(ee) %*% ee) / nrow(ee)
+#' lambda0 <- 0.2
+#' lambda1 <- 1
+#' lambda3 <- 100
+#' v1 <- matrix(data = (1:nlags), nrow = nlags, ncol = 1)
+#' v1 <- v1^((-2) * lambda1)
+#' v2 <- matrix(data = diag(solve(diag(diag(somega)))), ncol = 1)
+#' v3 <- kronecker(v1, v2)
+#' v3 <- (lambda0^2) * rbind(v3, (lambda3^2))
+#' v3 <- 1 / v3
+#' pP_sig <- diag(x = 1, nrow = nrow(v3), ncol = nrow(v3))
+#' diag(pP_sig) <- v3
+#' 
+#' # Confidence in long-run restriction priors
+#' pR_sig <-
+#'   array(data = 0,
+#'         dim = c(((nlags * ncol(y)) + 1),
+#'                 ((nlags * ncol(y)) + 1),
+#'                 ncol(y)))
+#' Ri <-
+#'   cbind(kronecker(matrix(data = 1, nrow = 1, ncol = nlags),
+#'                   matrix(data = c(1, 0), nrow = 1)),
+#'         0)
+#' pR_sig[, , 2] <- (t(Ri) %*% Ri) / 0.1
+#' 
+#' # Confidence in priors for D
+#' kappa1 <- matrix(data = 2, nrow = 1, ncol = ncol(y))
+#' 
+#' # Set graphical parameters
+#' par(cex.axis = 0.8, cex.main = 1, font.main = 1, family = "serif",
+#'     mfrow = c(2, 2), mar = c(2, 2.2, 2, 1), las = 1)
+#' 
+#' # Estimate the parameters of the model
+#' results1 <- 
+#'   BH_SBVAR(y = y, nlags = nlags, pA = pA, pP = pP, pP_sig = pP_sig,
+#'            pR_sig = pR_sig, kappa1 = kappa1, itr = itr, burn = burn,
+#'            thin = thin, acc_irf = acc_irf,
+#'            h1_irf = h1_irf, cri = cri)
+#' 
+#' # Plot impulse responses
+#' varnames <- colnames(USLMData)[2:3]
+#' shocknames <- c("Labor Demand","Labor Supply")
+#' fevd_results <- 
+#'   FEVD_Plots(results = results1, varnames = varnames,
+#'             shocknames = shocknames)
+FEVD_Plots <- function(results, varnames, shocknames = NULL, xlab = NULL, ylab = NULL, rel = TRUE) {
+  
+  #test arguments
+  test <- check_results(results = results, xlab = xlab, ylab = ylab)
+  if (test != "pass") {
+    stop(test)
+  }
+  if ((class(varnames) != "character") || (length(varnames) != dim(results$A)[2])) {
+    stop(paste("varnames: Must be a character vector containing the names of the endogenous variables", sep = ""))
+  }
+  if (is.null(shocknames)) {
+    shocknames <- varnames
+  }
+  if ((class(shocknames) != "character") || (length(shocknames) != dim(results$A)[2])) {
+    stop(paste("shocknames: Must be a character vector containing the names of the shocks", sep = ""))
+  }
+  
+  if (is.null(xlab)) {
+    xlab <- ""
+  }
+  if (is.null(ylab)) {
+    ylab <- ""
+  }
+  
+  if (!isTRUE(rel) & !isFALSE(rel)) {
+    stop("rel: Must be TRUE or FALSE.")
+  }
+  
+  FEVD <- results$FEVD
+  nvar <- dim(results$A)[1]
+  xticks <- floor(dim(FEVD)[1] / 4)
+  y <- results$y
+  ycolnames <- colnames(y)
+  FEVDcolnames <- dimnames(FEVD)[[2]]
+  
+  colors1 <- c("black", "red", "black", "red")
+  lty1 <- c(1, 2, 1, 2)
+  if (rel) {
+    colors1[] <- rep(x = "black", times = length(colors1))
+    lty1[] <- rep(x = 1, times = length(lty1))
+    ind1 <- rep(x = 0, times = length(varnames))
+    for (i in 1:length(varnames)) {
+      ind1[] <- grep(pattern = paste0("Res_", colnames(y)[i]), x = colnames(FEVD))
+      FEVD[, ind1, "50%"] <- FEVD[, ind1, "50%"] * matrix(data = rep(x = (100 / rowSums(FEVD[, ind1, "50%"])), times = ncol(y)), ncol = ncol(y), dimnames = list(NULL, FEVDcolnames[ind1]))
+    }
+    FEVD[, , c(1, 3)] <- FEVD[, , "50%"]
+  }
+  dimnames(FEVD)[[3]] <- rep(x = "50%", times = 3)
+  
+  #store results from fevd responses
+  fevd_results <- vector(mode = "list", length = (nvar * nvar))
+  for (j in 1:nvar) {
+    for (i in 1:nvar) {
+      #fevd responses
+      names(fevd_results)[((nvar * (j - 1)) + i)] <- FEVDcolnames[((nvar * (j - 1)) + i)]
+      fevd_results[[((nvar * (j - 1)) + i)]] <- FEVD[, ((nvar * (j - 1)) + i), ]
+      #fevd response plots
+      mat_ts <- stats::ts(cbind(0, fevd_results[[((nvar * (j - 1)) + i)]]))
+      colnames(mat_ts) <- c("Series1", "Series2", "Series3", "Series4")
+      stats::ts.plot(mat_ts, col = colors1, gpars = list(xlab = xlab, ylab = ylab, xaxs = "i", yaxs = "r", xaxt = "n", lty = lty1))
+      graphics::title(main = paste("Response of ", varnames[i], " to ", shocknames[j], sep = ""), col.main = "black")
+      graphics::axis(side = 1, at = seq(from = 1, to = nrow(mat_ts), by = xticks), labels = seq(from = 0, to = (nrow(mat_ts) - 1),by = xticks))
+    }
+  }
+  return(fevd_results)
+}
+
 #' Plot Historical Decompositions
 #' 
 #' Plot Historical Decompositions.
