@@ -719,6 +719,79 @@ arma::cube irf_estimates(const arma::cube& A_chain, const arma::cube& B_chain, c
   return IRF;
 }
 
+// Estimate Forecast Error Variance Decompositions and Process Raw Results.
+arma::cube fevd_estimates(const arma::cube& A_chain, const arma::cube& B_chain, const arma::cube& D_chain, const arma::uword pA_ncol, const arma::uword nlags, const arma::uword nsli, const arma::uword h1_irf, const bool acc_irf, const double ci) {
+  arma::uword nrow = (h1_irf + 1), ncol = (pA_ncol * pA_ncol);
+  
+  arma::cube FEVD_chain(nrow, ncol, nsli);
+  std::fill(FEVD_chain.begin(), FEVD_chain.end(), Rcpp::NumericVector::get_na());
+  
+  arma::mat H_draw(pA_ncol, pA_ncol), FEVD_draw((h1_irf + nlags), (pA_ncol * pA_ncol)), IRF_draw((h1_irf + nlags), (pA_ncol * pA_ncol)), Phi_draw(((pA_ncol * nlags) + 1), pA_ncol);
+  std::fill(H_draw.begin(), H_draw.end(), Rcpp::NumericVector::get_na());
+  std::fill(FEVD_draw.begin(), FEVD_draw.end(), Rcpp::NumericVector::get_na());
+  std::fill(IRF_draw.begin(), IRF_draw.end(), Rcpp::NumericVector::get_na());
+  std::fill(Phi_draw.begin(), Phi_draw.end(), Rcpp::NumericVector::get_na());
+  
+  for (arma::uword t = 0; t < nsli; ++t) {
+    //check for interruptions
+    if (t % 1024 == 0) {
+      Rcpp::checkUserInterrupt();
+    }
+    
+    //impulse responses (H_chain, IRF_chain, Phi_chain)
+    H_draw = arma::inv(A_chain.slice(t));
+    Phi_draw = B_chain.slice(t) * H_draw;
+    IRF_draw.fill(0.0);
+    for (arma::uword i = 0; i < pA_ncol; ++i) {
+      IRF_draw.submat((nrow - 1), (pA_ncol * i), (nrow - 1), ((pA_ncol * (i + 1)) - 1)) = H_draw.row(i);
+      for (arma::uword j = (nrow - 1); j >= 1; --j) {
+        IRF_draw.submat((j - 1), (pA_ncol * i), (j - 1), ((pA_ncol * (i + 1)) - 1)) = arma::vectorise(IRF_draw.submat(j, (pA_ncol * i), (j + nlags - 1), ((pA_ncol * (i + 1)) - 1)).t()).t() * Phi_draw(arma::span(0, ((pA_ncol * nlags) - 1)), arma::span::all);
+      }
+    }
+    if (acc_irf) {
+      for (arma::uword j = (nrow - 1); j >= 1; --j) {
+        IRF_draw.row((j - 1)) += IRF_draw.row(j);
+      }
+    }
+    
+    FEVD_draw.fill(0.0);
+    for (arma::uword i = 0; i < pA_ncol; ++i) {
+      FEVD_draw.submat((nrow - 1), (pA_ncol * i), (nrow - 1), ((pA_ncol * (i + 1)) - 1)) = (D_chain(i, i, t)) * (IRF_draw.submat((nrow - 1), (pA_ncol * i), (nrow - 1), ((pA_ncol * (i + 1)) - 1)) % IRF_draw.submat((nrow - 1), (pA_ncol * i), (nrow - 1), ((pA_ncol * (i + 1)) - 1)));
+      for (arma::uword j = (nrow - 1); j >= 1; --j) {
+        FEVD_draw.submat((j - 1), (pA_ncol * i), (j - 1), ((pA_ncol * (i + 1)) - 1)) = FEVD_draw.submat(j, (pA_ncol * i), j, ((pA_ncol * (i + 1)) - 1)) + ((D_chain(i, i, t)) * (IRF_draw.submat((j - 1), (pA_ncol * i), (j - 1), ((pA_ncol * (i + 1)) - 1)) % IRF_draw.submat((j - 1), (pA_ncol * i), (j - 1), ((pA_ncol * (i + 1)) - 1))));
+      }
+    }
+    
+    FEVD_chain.slice(t) = FEVD_draw(arma::span(0, (nrow - 1)), arma::span::all);
+  }
+  
+  //process FEVD_chain
+  arma::mat temp(1, nsli), temp1(nrow, nsli);
+  std::fill(temp.begin(), temp.end(), Rcpp::NumericVector::get_na());
+  std::fill(temp1.begin(), temp1.end(), Rcpp::NumericVector::get_na());
+  
+  arma::cube FEVD(nrow, ncol, 3);
+  std::fill(FEVD.begin(), FEVD.end(), Rcpp::NumericVector::get_na());
+  
+  double total = double (nsli);
+  arma::uword lb = arma::uword (std::round(total * (1.0 - ci)));
+  arma::uword ub = arma::uword (std::round((total * ci) - 1.0));
+  
+  for (arma::uword j = 0; j < ncol; ++j) {
+    
+    for (arma::uword i = 0; i < nrow; ++i) {
+      temp = FEVD_chain.tube(i, j);
+      temp1.row(i) = arma::sort(temp, "ascend", 1);
+    }
+    FEVD.slice(0).col(j) = arma::reverse(temp1.col(lb));
+    FEVD.slice(1).col(j) = arma::reverse(arma::median(temp1, 1));
+    FEVD.slice(2).col(j) = arma::reverse(temp1.col(ub));
+    
+  }
+  
+  return FEVD;
+}
+
 // Estimate Phi and Process Raw Results.
 arma::cube phi_estimates(const arma::cube& A_chain, const arma::cube& B_chain, const arma::uword pA_ncol, const arma::uword nlags, const arma::uword nsli, const double ci) {
   arma::uword nrow = ((pA_ncol * nlags) + 1), ncol = pA_ncol;
@@ -809,7 +882,7 @@ Rcpp::List deta_estimates(const arma::cube& A_chain, const arma::uword nsli, con
 //' @useDynLib BHSBVAR, .registration = TRUE
 //' @keywords internal
 // [[Rcpp::export]]
-Rcpp::List MAIN(const arma::mat& y1, const arma::mat& x1, const arma::mat& omega, const arma::mat& somega, const arma::uword nlags, const arma::cube& pA, const arma::cube& pdetA, const arma::cube& pH, const arma::mat& pP, const arma::mat& pP_sig, const arma::cube& pR_sig, const arma::mat& kappa1, const arma::mat& A_start, const arma::uword itr, const arma::uword burn, const arma::uword thin, const arma::mat& scale1, const arma::uword h1_irf, const bool acc_irf, const double ci, const Rcpp::StringVector& varnames, const Rcpp::Function& line_plot, const Rcpp::Function& acf_plot, const bool& rA, const bool& rB) {
+Rcpp::List MAIN(const arma::mat& y1, const arma::mat& x1, const arma::mat& omega, const arma::mat& somega, const arma::uword nlags, const arma::cube& pA, const arma::cube& pdetA, const arma::cube& pH, const arma::mat& pP, const arma::mat& pP_sig, const arma::cube& pR_sig, const arma::mat& kappa1, const arma::mat& A_start, const arma::uword itr, const arma::uword burn, const arma::uword thin, const arma::mat& scale1, const arma::uword h1_irf, const bool acc_irf, const double ci, const Rcpp::StringVector& varnames, const Rcpp::Function& line_plot, const Rcpp::Function& acf_plot, const bool& rA, const bool& rB, const bool& rD) {
   arma::uword pA_ncol = pA.n_cols, B_nrow = ((y1.n_cols * nlags) + 1);
   double totals = double (itr - burn), totalt = double (thin), y1_nrow = double (y1.n_rows);
   arma::uword nsli = arma::uword (std::floor((totals / totalt)));
@@ -817,7 +890,7 @@ Rcpp::List MAIN(const arma::mat& y1, const arma::mat& x1, const arma::mat& omega
   arma::field<arma::cube> list1 = MH(y1, x1, nlags, omega, somega, pA, pdetA, pH, pP, pP_sig, pR_sig, kappa1, A_start, itr, burn, scale1);
   double accept_rate = list1(0)(0, 0, 0);
   
-  arma::cube A_chain(pA_ncol, pA_ncol, nsli), B_chain(B_nrow, pA_ncol, nsli), zeta_chain(pA_ncol, pA_ncol, nsli);
+  arma::cube A_chain(pA_ncol, pA_ncol, nsli), B_chain(B_nrow, pA_ncol, nsli), zeta_chain(pA_ncol, pA_ncol, nsli), D_chain(pA_ncol, pA_ncol, nsli, arma::fill::zeros);
   std::fill(A_chain.begin(), A_chain.end(), Rcpp::NumericVector::get_na());
   std::fill(B_chain.begin(), B_chain.end(), Rcpp::NumericVector::get_na());
   std::fill(zeta_chain.begin(), zeta_chain.end(), Rcpp::NumericVector::get_na());
@@ -858,6 +931,7 @@ Rcpp::List MAIN(const arma::mat& y1, const arma::mat& x1, const arma::mat& omega
     for (arma::uword i = 0; i < pA_ncol; ++i) {
       Dinv_draw(i, i) = R::rgamma((kappastar(i, i)), (1.0 / (taustar(i, i))));
     }
+    D_chain.slice(t) = arma::inv(Dinv_draw);
     
     for (arma::uword i = 0; i < pA_ncol; ++i) {
       M = temp0.slice(i) * arma::randn(B_nrow, pA_ncol) * arma::inv(arma::sqrt(Dinv_draw));
@@ -882,12 +956,12 @@ Rcpp::List MAIN(const arma::mat& y1, const arma::mat& x1, const arma::mat& omega
   Rcpp::List list2 = deta_estimates(A_chain, nsli, pdetA, line_plot, acf_plot, ci);
   Rcpp::List list3 = h_estimates(A_chain, pA_ncol, nsli, pH, line_plot, acf_plot, ci);
   
-  Rcpp::List list4(25);
+  Rcpp::List list4(28);
   list4.names() = 
     Rcpp::CharacterVector(
       {"accept_rate", "y", "x", "pA", "pdetA", "pH", "pP", "pP_sig", "pR", "pR_sig",
-       "tau1", "kappa1", "A_start", "A", "detA", "H", "B", "Phi", "HD", "IRF",
-       "A_den", "detA_den", "H_den", "A_chain", "B_chain"}
+       "tau1", "kappa1", "A_start", "A", "detA", "H", "B", "Phi", "D", "HD", "IRF", "FEVD",
+       "A_den", "detA_den", "H_den", "A_chain", "B_chain", "D_chain"}
     );
   
   list4["accept_rate"] = accept_rate;
@@ -912,9 +986,11 @@ Rcpp::List MAIN(const arma::mat& y1, const arma::mat& x1, const arma::mat& omega
   list4["H"] = list3["H"];
   list4["B"] = results_function(B_chain, ci);
   list4["Phi"] = phi_estimates(A_chain, B_chain, pA_ncol, nlags, nsli, ci);
+  list4["D"] = results_function(D_chain, ci);
   
   list4["HD"] = hd_estimates(A_chain, B_chain, y1, x1, pA_ncol, nlags, nsli, ci);
   list4["IRF"] = irf_estimates(A_chain, B_chain, pA_ncol, nlags, nsli, h1_irf, acc_irf, ci);
+  list4["FEVD"] = fevd_estimates(A_chain, B_chain, D_chain, pA_ncol, nlags, nsli, h1_irf, acc_irf, ci);
   
   list4["A_den"] = den_function(A_chain, pA);
   list4["detA_den"] = list2["detA_den"];
@@ -926,6 +1002,9 @@ Rcpp::List MAIN(const arma::mat& y1, const arma::mat& x1, const arma::mat& omega
   if (rB) {
     list4["B_chain"] = B_chain;
   }
+  if (rD) {
+    list4["D_chain"] = D_chain;
+  }
   
   return list4;
 }
